clear; clc
% options     
    % COM PORT
    % Select the COM port for the connection to the E-Nose
    comPort = '/dev/ttyUSB0';

%------------------------
close all; clc
addpath('lib');
load("allTemperatures30-400.mat")
load("selectedFeatures.mat")
load("ecocmodel.mat");
filetime = datetime;

if exist('Xmean','var')
    standardize = true;
else
    standardize = false;
end


% save data prompt
while true
prompt = 'Export data to test in csv? Enter the filename, o/w press enter:\n';
fnt = input(prompt,'s');

if ~contains(fnt, ["/","\","*",":","?",string('"'),"<",">","|"])
    break;
end
disp('Do not use one of the following characters: /\*:?"<>|');
fnt='';
end

% measurement configuration parameters are set
FeatureTempOffset = 30; % Feature 1 corresponds to 30�C and so on
heatingDuration = 100;
% for faster measurement, the temperatures are measured in ascending order
[TempControl, SortIndex] = sort(selectedFeatures);
TempControl = TempControl + (FeatureTempOffset-1);

classes = unique(Ycategories);
numClasses = numel(unique(Ycategories));
warning('off','MATLAB:serial:fscanf:unsuccessfulRead');


% _hsv()_ is used to plot the markers to the different classes in their 
% respective colors.
fgh = figure;
colormap(zeros(numClasses, 3));
hsv_t = hsv;
colormap(hsv_t);
hold on

noseSerialObj = setupSerial(comPort);

% plot hyperplanes only if coding design is set to ordinal
if (strcmp(coding,'ordinal'))
    w = []; w0 = [];
    % plot standardized data with hyperplanes if coding design == ordinal
    for i = 1:numel(Mdl.BinaryLearners)
        w = [w, Mdl.BinaryLearners{i}.Beta];
        w0 = [w0, Mdl.BinaryLearners{i}.Bias];   
    end
    plotHyperplanes(Ycategories, hsv_t, fgh, Mdl.ClassNames, w, w0);
    drawnow
end

% the control string is constructed
% it is specified as follows:
% 'TEMP:' followed by the temperatures to measure in the order of their measurement
% followed by 'HTDR:' followed by the desired heating duration of the sensor
% followed by ';;' to signal the end of the string
% example: TEMP:100,200,300,400;HTDR:200;;

StrControl = ['TEMP:' num2str(round(TempControl(1)))];
 for i=2:length(TempControl)
      StrControl = [StrControl ',' num2str(round(TempControl(i))) ]  ;
 end
 StrControl = [StrControl, ';HTDR:', num2str(heatingDuration), ';;'];

 
 newCheckLabelValue = false;
 newEnvValue = false;
 newSubstValue = false;
 
% initializations for data export
X = [];
Y = [];
measurementNo = 0;
 
while true
   pause(1);
   serialInput = fscanf(noseSerialObj, '%s');
   
   % the script waits for inputs by the nose, then classifies the input, so that it can react properly afterwards
   if ~isempty(serialInput)
       switch serialInput(1)
	   % the input is classified by the first letter of the input string
           case 'S'
               if serialInput(2) ~= 'o'  % 'SoftWDTreset...'
                    newSubstValue = true;
               end
           case 'E'
               newEnvValue = true;
           case 'I'
		   % a new nose was connected and waits for the input string.
		   % the string is split into multiple chunks and sent piece after piece, to avoid overflowing the
		   % serial output buffer size of the matlab client
               StrControlChunk = StrControl;
               disp("Transfering control string ...");
                while (~isempty(StrControlChunk))
                     until = min([noseSerialObj.OutputBufferSize, length(StrControlChunk)]);
                     fprintf(noseSerialObj,'%s',StrControlChunk(1:until));
                     pause(1);
                     if (until < length(StrControlChunk))
                        StrControlChunk = StrControlChunk(until+1:end);
                     else
                         break;
                     end
                end
           case 'O'
               disp(serialInput);
       end
   end
   
   if newEnvValue
		% a new environment measurement arrived.
        newEnvValue = false;
        StrEnv = serialInput;
        
        while (StrEnv(end) ~= ';')
			% receive all chunks of the environment measurement
            StrEnv = [StrEnv, fscanf(noseSerialObj, '%s')];
        end
        
        disp("New Environment Measurement Dataset arrived:");
        disp(StrEnv);
        
		% save the data in a vector
        env=str2num(StrEnv(5:end));
        beep
   end
   if newSubstValue
		% a new substance measurement arrived.
        newSubstValue = false;
        StrSubst = serialInput;
        
        while (StrSubst(end) ~= ';')
			% receive all chunks of the substance measurement
            StrSubst = [StrSubst, fscanf(noseSerialObj, '%s')];
        end
        
        disp("New Substance Measurement Dataset arrived:");
        disp(StrSubst);
		
		% save the data in a vector
        subst = str2num(StrSubst(7:end));
		
		% calculate resulting data from substance data and 
		% previously sent environment data
        res = (subst - env) ./ env;
        
        % standardize data with model parameters
        if standardize
            res = (res - Xmean(selectedFeatures)) ./ Xstd(selectedFeatures);
        end
        
		% for faster measurement, the temperatures were measured in ascending order. They are
		% now reversed into the former order to fit the SVM's feature order.
        x = unsort(res,SortIndex);
		
		% performing the prediction
        curClass = predict(Mdl,x);
		
		% sending the predicted class to the Iot kit for display
        fprintf(noseSerialObj,'%s', strcat("  ", curClass));
        
		% plotting the result with matlab
        plot(x(1), x(2), 's', 'Color', hsv_t(curClass == classes,:));
        drawnow
        
        % save data to export
        time = datetime;
        X = [X;array2timetable(x,'RowTimes',time)];
        
        % export data if so wished
         if ~isempty(fnt)
             measurementNo = measurementNo + 1;
             saveTrainingSVM(fnt, filetime, X,...
                Y, measurementNo);
         end
   end
end